clear all;

%%% -   Explaination    -   %%%
% Code used for loading data from TDMS file
% Assigns data to variables
%%%     %%%     %%%     %%%

%Add path with file for Lanczos filter
addpath(genpath('C:\marinlab')); %path to marinlab, access Lanczos filter
%Specify folder path for simulation
folder = 'C:\project_number\';
hlcc = '\HLCC_';
defrag = '_defrag.tdms';

%Speciy which simulation to load into workspace:
file = 'file_number';

%Construct filepath for relevant simulation: 
file_path = string(folder) + string(file) + string(hlcc) + string(file) + string(defrag);

% Read the file using the tdmsread function 
data = tdmsread(file_path);
data_size = size(data); %variable for amount of data in TDMS file

%Iterate thorugh data to find correct input and output cells
for i=1:data_size(2)
    a = data{1,i};
    b = size(a);
    if (b(2) == 7)  %The input to Observer FMU has 7 columns
        input = a;  
    elseif b(2)==13 %The output from Observer FMU has 13 columns     
        output = a; 
    elseif b(2)==23 %The parameters are specified in cell of 23 columns
        param = a;  
    end
end

%Operation to ensure that the input and output vectors are of equal length
input_size = size(input);
output_size = size(output);

%Some simulations suffers from input having one row more than output
while output_size(1) > input_size(1)
    output = output(1:end-1,:);
    output_size = size(output_size);
end

%Perform operation to remove static data before and after simulation
eta_MoCap_surge = input.("eta1_MoCap_m_ []")(:);    %load surge vector
first_val = eta_MoCap_surge(1);
fin_val = eta_MoCap_surge(end);

idx = 1;
r_idx = length(eta_MoCap_surge);

%Find the begining of simulation by searching for first change in value:
while idx < length(eta_MoCap_surge) && eta_MoCap_surge(idx) == first_val
    idx = idx + 1;
end

%Repeat for the final value:
while r_idx>1 && eta_MoCap_surge(r_idx) == fin_val
    r_idx = r_idx - 1;
end

idx = idx + 100;    %Apply safety margin for simulation start 
compWindow = idx:r_idx; %Define compilation window 

%Assign input variables in the defined complilation window
eta_MoCap_surge = eta_MoCap_surge(compWindow);
eta_MoCap_sway = input.("eta2_MoCap_m_ []")(compWindow);
eta_MoCap_yaw = input.("eta6_MoCap_deg_ []")(compWindow);
tau_ref_surge = input.("tau1_ref_N_ []")(compWindow);
tau_ref_sway = input.("tau2_ref_N_ []")(compWindow);
tau_ref_yaw = input.("tau6_ref_Nm_ []")(compWindow);

%Assign input variables in the defined complilation window
%Begin by defining time from output vector (NB! in UNIX-format)
my_time = output.("TIME [s]")(compWindow); 
first_time = output.("TIME [s]")(1);
time = my_time - first_time; %convert from unix

%Then assign rest of observer output variables in compilation window
eta_WF_surge = output.("eta1_WF_m_ []")(compWindow);
eta_WF_sway = output.("eta2_WF_m_ []")(compWindow);
eta_WF_yaw = output.("eta6_WF_deg_ []")(compWindow);
eta_LF_surge = output.("eta1_LF_m_ []")(compWindow);
eta_LF_sway = output.("eta2_LF_m_ []")(compWindow);
eta_LF_yaw = output.("eta6_LF_deg_ []")(compWindow);
nu_LF_surge = output.("nu1_LF_m_s_ []")(compWindow);
nu_LF_sway = output.("nu2_LF_m_s_ []")(compWindow);
nu_LF_yaw = output.("nu6_LF_deg_s_ []")(compWindow);
bias_surge = output.("bias1_N_ []")(compWindow);
bias_sway = output.("bias2_N_ []")(compWindow);
bias_yaw = output.("bias6_Nm_ []")(compWindow);

%%% - Define offline filter - %%% 
scale = 40;     %model scaling factor
T = time(end);
cutOffPer = 30/sqrt(scale); %the cutoff wave period for ideal offline filter
dt = mean(gradient(time))   %mean change in time as dt, can vary for simulations
stdTime = std(gradient(time))   %Show standard deviation as failsafe
f1 = filt.Lanczos(2*pi/cutOffPer,3,'lo',dt); %lowpass filter using FFT from marinlab, SINTEF Environment

%%% - Apply offline filter - %%% 
Nfilt = f1*eta_MoCap_surge;  %XPOS
Efilt = f1*eta_MoCap_sway;  %YPOS
psiFilt = f1*eta_MoCap_yaw;%YAW

%%% - Mooring reference forces - %%% 
Fx = tau_ref_surge;
Fy = tau_ref_sway;
Mz = tau_ref_yaw;

%%% - Derive offline velocity estimates - %%% 
velGlobFilt = [gradient(Nfilt)/dt gradient(Efilt)/dt gradient(psiFilt)/dt];
velLocFilt = zeros(length(my_time),3);
for idx=1:length(my_time)
        %rotate to body-frame
    velLocFilt(idx,:) = (Rzyx_(0,0,eta_MoCap_yaw(idx)*pi/180).'*velGlobFilt(idx,:).').';
end

%%% - Check and display frequency relation defined by Fossen - %%% 
K3 = param.("K3 []");
K4 = param.("K4_1 []");
Tb = param.("Tb []");
K3_K4 = (K3/K4);
omega_0 = param.("Tp []");
omega_c = param.("cutOffWFObs []");
q = (Tb<K3_K4<omega_0<omega_c);
fprintf('1/T_i: %f << K3/K4: %f < w_0: %f < wc: %f \n is: %i \n', Tb, K3_K4, omega_0, omega_c, q);
